<?php

if (!defined('ABSPATH')) {
    exit;
}


function Load_dpay_Gateway()
{

    if (!function_exists('Woocommerce_Add_dpay_Gateway') && class_exists('WC_Payment_Gateway') && !class_exists('WC_dpay')) {


        add_filter('woocommerce_payment_gateways', 'Woocommerce_Add_dpay_Gateway');

        function Woocommerce_Add_dpay_Gateway($methods)
        {
            $methods[] = 'WC_dpay';
            return $methods;
        }

        add_filter('woocommerce_currencies', 'add_IRcurrency');

        function add_IRcurrency($currencies)
        {
            $currencies['DLR'] = __('Dollar', 'woocommerce');
            return $currencies;
        }

        add_filter('woocommerce_currency_symbol', 'add_IRcurrencysymbol', 10, 2);

        function add_IRcurrencysymbol($currency_symbol, $currency)
        {
            switch ($currency) {
                case 'DLR':
                    $currency_symbol = '$';
                    break;
            }
            return $currency_symbol;
        }

        class WC_dpay extends WC_Payment_Gateway
        {


            private $merchantCode;
            private $failedMassage;
            private $successMassage;

            public function __construct()
            {

                $this->id = 'WC_dpay';
                $this->method_title = __('D-pay Cryptocurrency Payment Gateway', 'woocommerce');
                $this->method_description = __('D-pay payment gateway settings for WooCommerce', 'woocommerce');
                $this->icon = apply_filters('WC_dpay_logo', WP_PLUGIN_URL . '/' . plugin_basename(__DIR__) . '/assets/images/logo.png');
                $this->has_fields = false;

                $this->init_form_fields();
                $this->init_settings();

                $this->title = $this->settings['title'];
                $this->description = $this->settings['description'];

                $this->merchantCode = $this->settings['merchantcode'];

                $this->successMassage = $this->settings['success_massage'];
                $this->failedMassage = $this->settings['failed_massage'];

                if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                    add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
                } else {
                    add_action('woocommerce_update_options_payment_gateways', array($this, 'process_admin_options'));
                }

                add_action('woocommerce_receipt_' . $this->id . '', array($this, 'Send_to_dpay_Gateway'));
                add_action('woocommerce_api_' . strtolower(get_class($this)) . '', array($this, 'Return_from_dpay_Gateway'));


            }

            public function init_form_fields()
            {
                $this->form_fields = apply_filters('WC_dpay_Config', array(
                        'base_config' => array(
                            'title' => __('Basic Settings', 'woocommerce'),
                            'type' => 'title',
                            'description' => '',
                        ),
                        'enabled' => array(
                            'title' => __('Activation/Deactivation', 'woocommerce'),
                            'type' => 'checkbox',
                            'label' => __('D-pay Gateway Activation', 'woocommerce'),
                            'description' => __('For D-pay Payment Gateway Ativation; Please Check the Checkbox', 'woocommerce'),
                            'default' => 'yes',
                            'desc_tip' => true,
                        ),
                        'title' => array(
                            'title' => __('Payment Gateway Title', 'woocommerce'),
                            'type' => 'text',
                            'description' => __('The title of the payment gateway that is displayed to the customer during the purchase', 'woocommerce'),
                            'default' => __('D-pay Cryptocurrency Payment Gateway', 'woocommerce'),
                            'desc_tip' => true,
                        ),
                        'description' => array(
                            'title' => __('Payment Gateway Description', 'woocommerce'),
                            'type' => 'text',
                            'desc_tip' => true,
                            'description' => __('The description that will be displayed during the purchase for the payment gateway to the customer', 'woocommerce'),
                            'default' => __('Payment Using Cryptocurrency for Online Stores', 'woocommerce')
                        ),
                        'account_config' => array(
                            'title' => __('Account Settings', 'woocommerce'),
                            'type' => 'title',
                            'description' => '',
                        ),
                        'merchantcode' => array(
                            'title' => __('Merchant Code', 'woocommerce'),
                            'type' => 'text',
                            'description' => __('D-pay Merchant Code', 'woocommerce'),
                            'default' => '',
                            'desc_tip' => true
                        ),
                        'dpaywebgate' => array(
                            'title' => __('D-pay Activation', 'woocommerce'),
                            'type' => 'checkbox',
                            'label' => __('For D-pay Gateway Activation Please Check the Checkbox', 'woocommerce'),
                            'description' => __('D-pay Direct Payment Gateway', 'woocommerce'),
                            'default' => '',
                            'desc_tip' => true,
                        ),
                        'payment_config' => array(
                            'title' => __('Payment Settings', 'woocommerce'),
                            'type' => 'title',
                            'description' => '',
                        ),
                        'success_massage' => array(
                            'title' => __('Successful Transaction Message', 'woocommerce'),
                            'type' => 'textarea',
                            'description' => __('Enter the message that you want to be displayed to the user after successful payment. You can also use the short code {transaction_id} to display the Transaction ID.', 'woocommerce'),
                            'default' => __('Thank you For your Purchase. Your Order Completed Successfully ', 'woocommerce'),
                        ),
                        'failed_massage' => array(
                            'title' => __('Failed Transaction Message', 'woocommerce'),
                            'type' => 'textarea',
                            'description' => __('Enter the message you want to display to the user after an unsuccessful payment. You can also use the {fault} shortcode to display the reason for the error that occurred.', 'woocommerce'),
                            'default' => __('Your payment has failed. Please try again or contact the website administrator.', 'woocommerce'),
                        ),
                    )
                );
            }

            public function process_payment($order_id)
            {
                $order = new WC_Order($order_id);
                return array(
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true)
                );
            }

            /**
             * @param $action (PaymentRequest, )
             * @param $params string
             *
             * @return mixed
             */
            public function SendRequestTodpay($action, $params)
            {
                try {
                    $number_of_connection_tries = 5;
                    $response = null;
                    while ($number_of_connection_tries > 0) {
                        $response = wp_safe_remote_post('https://api.d-pay.io/order/'.$action, array(
                            'body' => $params,
                            'headers' => array(
                                'Content-Type' => 'application/json',
                                'Content-Length' =>   strlen($params) ,
                                'User-Agent' => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36 Edg/105.0.1343.33'
                            )
                        ));
                        // print_r($response);
                        if (is_wp_error($response)) {
                            $number_of_connection_tries--;
                             sprintf(__('Error While Sending To Blockchain: %s', 'woocommerce'), $Message); 
                            continue;
                        } else {
                            break;
                        }
                    }

                    $body = wp_remote_retrieve_body($response);

                    return json_decode($body, true);
                } catch (Exception $ex) {
                    return false;
                }
            }

            public function Send_to_dpay_Gateway($order_id)
            {


                global $woocommerce;
                $woocommerce->session->order_id_dpay = $order_id;
                $order = new WC_Order($order_id);
                $currency = $order->get_currency();
                $currency = apply_filters('WC_dpay_Currency', $currency, $order_id);


                $form = '<form action="" method="POST" class="zarinpal-checkout-form" id="zarinpal-checkout-form">
						<input type="submit" name="zarinpal_submit" class="button alt" id="zarinpal-payment-button" value="' . __('payment', 'woocommerce') . '"/>
						<a class="button cancel" href="' . $woocommerce->cart->get_checkout_url() . '">' . __('back', 'woocommerce') . '</a>
					 </form><br/>';
                $form = apply_filters('WC_dpay_Form', $form, $order_id, $woocommerce);

                do_action('WC_dpay_Gateway_Before_Form', $order_id, $woocommerce);
                echo $form;
                do_action('WC_dpay_Gateway_After_Form', $order_id, $woocommerce);


                $Amount = (int)$order->order_total;
  
                $strToLowerCurrency = strtolower($currency);

         
                $Amount = apply_filters('woocommerce_order_amount_total_dpay_gateway', $Amount, $currency);

                $CallbackUrl = add_query_arg('wc_order', $order_id, WC()->api_request_url('WC_dpay'));

                $products = array();
                $order_items = $order->get_items();
                foreach ($order_items as $product) {
                    $products[] = $product['name'] . ' (' . $product['qty'] . ') ';
                }
                $products = implode(' - ', $products);

                $Description = 'Purchase with order ID: ' . $order->get_order_number() . ' | Customer : ' . $order->billing_first_name . ' ' . $order->billing_last_name ;
                $Mobile = get_post_meta($order_id, '_billing_phone', true) ?: '-';
                $Email = $order->billing_email;
                $Payer = $order->billing_first_name . ' ' . $order->billing_last_name;
                $ResNumber = (int)$order->get_order_number();

                //Hooks for iranian developer
                $Description = apply_filters('WC_dpay_Description', $Description, $order_id);
                $Mobile = apply_filters('WC_dpay_Mobile', $Mobile, $order_id);
                $Email = apply_filters('WC_dpay_Email', $Email, $order_id);
                $Payer = apply_filters('WC_dpay_Paymenter', $Payer, $order_id);
                $ResNumber = apply_filters('WC_dpay_ResNumber', $ResNumber, $order_id);
                do_action('WC_dpay_Gateway_Payment', $order_id, $Description, $Mobile);
                $Email = !filter_var($Email, FILTER_VALIDATE_EMAIL) === false ? $Email : '';
                $Mobile = preg_match('/^09[0-9]{9}/i', $Mobile) ? $Mobile : '';

                $data['id_order'] =  $order_id;
                $data['merchant_code'] =  $this->merchantCode;
                $data['amount'] =  $Amount;
                $data['callback'] =  $CallbackUrl;
                $data['email'] =  $Email;
                $data['mobail'] =  $Mobile;

                $result = $this->SendRequestTodpay('add', json_encode($data));
                if ($result['status'] == "20") {
                    header('Location: ' . $result['backlink']);
                    exit;
                }
                else {
                  
                    $Message =  $result['message'];
                    $Fault = '';

                    $Note = sprintf(__('Error sending to blockchain : %s', 'woocommerce'), $Message);
                    $Note = apply_filters('WC_dpay_Send_to_Gateway_Failed_Note', $Note, $order_id, $Fault);
                    $order->add_order_note($Note);


                    $Notice = sprintf(__('An error occurred while connecting to the blockchain  : <br/>%s', 'woocommerce'), $Message);
                    $Notice = apply_filters('WC_dpay_Send_to_Gateway_Failed_Notice', $Notice, $order_id, $Fault);
                    if ($Notice) {
                        wc_add_notice($Notice, 'error');
                    }

                    do_action('WC_dpay_Send_to_Gateway_Failed', $order_id, $Fault);
                }
            }


            public function Return_from_dpay_Gateway()
            {

                $Authority = isset($_REQUEST['Authority']) ? sanitize_text_field($_REQUEST['Authority'] ): '';

                global $woocommerce;


                if (isset($_GET['wc_order'])) {
                    $order_id = sanitize_text_field($_GET['wc_order']) ;
                }
                 else {
                    $order_id = $woocommerce->session->order_id_dpay;
                    unset($woocommerce->session->order_id_dpay);
                }

                if ($order_id) {
                    $order = new WC_Order($order_id);
                    $currency = $order->get_currency();
                    $currency = apply_filters('WC_dpay_Currency', $currency, $order_id);

                    if ($order->status !== 'completed') {

                        $MerchantCode = $this->merchantCode;

                        if ($_GET['status'] === '20') {
                            $MerchantID = $this->merchantCode;
                            $Amount = (int)$order->order_total;
                            $Amount = apply_filters('woocommerce_order_amount_total_cryptocurrency_gateways_before_check_currency', $Amount, $currency);
                            $strToLowerCurrency = strtolower($currency);

                            $data = array("MerchantID" => $MerchantID, "Authority" => $Authority, "Amount" => $Amount);
                            $result = $this->SendRequestTodpay('verify', json_encode($data));

                            if ($result['status'] == "20") {
                                $Status = 'completed';
                                $Transaction_ID = $MerchantID;
                                $Fault = '';
                                $Message = '';
                            } 
                            elseif ($result['status'] == "21") {
                                $Message = 'This transaction has already been confirmed.';
                                $Transaction_ID = $MerchantID;
                                $Notice = wpautop(wptexturize($Message));
                                wp_redirect(add_query_arg('wc_status', 'success', $this->get_return_url($order)));
                                exit;
                            }
                             else {
                                $Status = 'failed';
                                $Fault = $result['message'];
                                $Message = $result['message'];
                            }
                        }
                         else {
                            $Status = 'failed';
                            $Fault = '';
                            $Message = 'The transaction was not completed.';
                        }

                        if ($Status === 'completed' && isset($Transaction_ID) && $Transaction_ID !== 0) {
                            update_post_meta($order_id, '_transaction_id', $Transaction_ID);



                            $order->payment_complete($Transaction_ID);
                            $woocommerce->cart->empty_cart();

                            $Note = sprintf(__('Transaction Completed Successfully.<br/> Transaction ID: %s', 'woocommerce'), $Transaction_ID);
                            $Note = apply_filters('WC_dpay_Return_from_Gateway_Success_Note', $Note, $order_id, $Transaction_ID);
                            if ($Note)
                                $order->add_order_note($Note, 1);


                            $Notice = wpautop(wptexturize($this->successMassage));

                            $Notice = str_replace('{transaction_id}', $Transaction_ID, $Notice);

                            $Notice = apply_filters('WC_dpay_Return_from_Gateway_Success_Notice', $Notice, $order_id, $Transaction_ID);
                            if ($Notice)
                                wc_add_notice($Notice, 'success');

                            do_action('WC_dpay_Return_from_Gateway_Success', $order_id, $Transaction_ID);

                            wp_redirect(add_query_arg('wc_status', 'success', $this->get_return_url($order)));
                            exit;
                        }

                        if (($Transaction_ID && ($Transaction_ID != 0))) {
                            $tr_id = ('<br/>Transaction ID : ' . $Transaction_ID);
                        } else {
                            $tr_id = '';
                        }

                        $Note = sprintf(__('Error While returning from Blockchain: %s %s', 'woocommerce'), $Message, $tr_id);

                        $Note = apply_filters('WC_dpay_Return_from_Gateway_Failed_Note', $Note, $order_id, $Transaction_ID, $Fault);
                        if ($Note) {
                            $order->add_order_note($Note, 1);
                        }

                        $Notice = wpautop(wptexturize($this->failedMassage));

                        $Notice = str_replace(array('{transaction_id}', '{fault}'), array($Transaction_ID, $Message), $Notice);
                        $Notice = apply_filters('WC_dpay_Return_from_Gateway_Failed_Notice', $Notice, $order_id, $Transaction_ID, $Fault);
                        if ($Notice) {
                            wc_add_notice($Notice, 'error');
                        }

                        do_action('WC_dpay_Return_from_Gateway_Failed', $order_id, $Transaction_ID, $Fault);

                        wp_redirect($woocommerce->cart->get_checkout_url());
                        exit;
                    }

                    $Transaction_ID = get_post_meta($order_id, '_transaction_id', true);

                    $Notice = wpautop(wptexturize($this->successMassage));

                    $Notice = str_replace('{transaction_id}', $Transaction_ID, $Notice);

                    $Notice = apply_filters('WC_dpay_Return_from_Gateway_ReSuccess_Notice', $Notice, $order_id, $Transaction_ID);
                    if ($Notice) {
                        wc_add_notice($Notice, 'success');
                    }

                    do_action('WC_dpay_Return_from_Gateway_ReSuccess', $order_id, $Transaction_ID);

                    wp_redirect(add_query_arg('wc_status', 'success', $this->get_return_url($order)));
                    exit;
                }

                $Fault = __('Order Id Does not Exist.', 'woocommerce');
                $Notice = wpautop(wptexturize($this->failedMassage));
                $Notice = str_replace('{fault}', $Fault, $Notice);
                $Notice = apply_filters('WC_dpay_Return_from_Gateway_No_Order_ID_Notice', $Notice, $order_id, $Fault);
                if ($Notice) {
                    wc_add_notice($Notice, 'error');
                }

                do_action('WC_dpay_Return_from_Gateway_No_Order_ID', $order_id, '0', $Fault);

                wp_redirect($woocommerce->cart->get_checkout_url());
                exit;
            }

        }

    }
}

add_action('plugins_loaded', 'Load_dpay_Gateway', 0);
